/*
(690)   Wed 5 Jul 95  7:47p
By: Nick Hammond
To: Jan Doggen
Re: Do not scroll
St:
------------------------------------------------------------
@MSGID: 3:620/243.0 2ffc693e
@PID: BWRA 3.10a Beta [Reg]
@TID: GE 1.12.b2+
Hi Jan,

 JD> How can I prevent MEMOEDIT() from scrolling?
 JD> I want to limit the amount of text to the visible window.

The following is a library function which might help.  The only
external functions not in the clipper library are enhvideo() and
stdvideo() which simply return appropriate colour strings, and
errorwindow() which pops up a window with an error message for about
half a second.

The documentation at the front is designed to work with a modified
version of the nanforum toolkit documentation generator but should
be decypherable.

Regards, Nick.
*/

/*********************************************************************
MEMOGET.PRG -- get reader for memo fields

Author....      NDH
Last Modified:  07AUG93

$DOC$

$FUNCNAME$
   MEMOREADER
$CATEGORY$
   Function Reference
$ONELINER$
   Get reader for memo fields
$SYNTAX$
   function memoreader(oGet, nTop, nBot, nLft, nRgt, [nLimit])
             => NIL
$ARGUMENTS$
   oGet: get object associated with field
   nTop, nBot, nLft, nRgt:  Screen coordinates
   nLimit: optional value to limit number of lines accepted
           Defaults to the number of lines in the specified edit
           window (nBottom - nTop - 1).  Zero allows unlimited lines
$RETURNS$
   NIL
$DESCRIPTION$
   Memoreader is an alternative get reader function allowing
   the use of the get system to accept memo fields.  It is
   invoked by the "send reader" clause of the get command.
   Alternatively it can be specifically assigned to the
   get:reader code block for the particular get object.

   The function in turn invokes the standard Memoedit within the
   specified window, limiting the number of lines which can be
   entered to the vertical size of the window (unless lLimit is
   set to >F>).  Escape from memoedit is by the Up arrow on the
   top line, the Down arrow on the bottom line, or the Page Down
   or [F10] keys anywhere in the field.

   The function correctly sets the value returned by updated()
$EXAMPLES$
   The following example displays current contents of the memo
   field in enhanced video at the appropriate coordinates, then
   invokes the get system to accept new or amended data:

   // display the current memo field contents
   setcolor(enhvideo())
   memoedit(memfld, top, left, bottom, right, .F., .F.)
   setcolor(stdvideo())
   // get updates
   @top, left - 12 say "Memofield: " ;
                get memfld ;
                send reader := {|x|memoreader(x, tp, lf, bt, rt)}

   The following line of code shows the alternative method of specific
   allocation of the function to the get reader code block:

   atail(getlist):reader := {|x|memoreader(x, tp, lf, bt, rt)}

$SEEALSO$

$INCLUDE$

$END$
*********************************************************************/

#include "FBN.CH"
#include "INKEY.CH"
#include "GETEXIT.CH"

static updated
static maxline
static maxcol

function memoreader(get, top, left, bottom, right, limit)
// memo reader use for memoedit in gets
local buf                               // holds the new field
local e_state := GE_DOWN                // the exit state for the get
limit := iif(limit == NIL, bottom - top + 1, limit)
// reset local updated flag
updated := .F.
// default maxline and maxcol to edges of window
maxline := iif(limit > 0, limit, 9999)
maxcol := right - left - 1
// can't remember what this was for
keyboard chr(1)
// call the contents of the "valid" clause
if (getprevalidate(get))
   // if OK set colour and focus, then invoke memoedit
   setcolor(enhvideo())
   get:setfocus()
   buf := memoedit(get:varget(), top, left, bottom, right, ;
                        .T., "_getmemokeys", right - left + 1)
   // quit if user pressed [Esc]
   if (lastkey() == K_ESC)
      get:undo()
      updated := .F.
      e_state := GE_ESCAPE
   // else save the buffer and set exit state depending on key
   else
      get:varput(buf)
      if (nextkey() == K_UP)
         inkey()
         e_state := GE_UP
      elseif (nextkey() == K_DOWN)
         inkey()
         e_state := GE_DOWN
      elseif (nextkey() == K_PGDN)
         inkey()
         e_state := GE_WRITE
      endif
   endif
   // redisplay changed field, set clipper updated() function
   memoedit(get:varget(), top, left, bottom, right, .F., .F.)
   if (updated)
      updated(.T.)
   endif
   // final tidying up
   get:exitstate := e_state
   get:killfocus()
   setcolor(stdvideo())
endif
return NIL


function _getmemokeys(mode, line, column)
// udf to handle idle state or unassigned key in memoedit
// must be public because memoedit calls it as a macro (ecch!)
// exits on [Up] in top line or [PgDn] or [F10] anywhere
static prevline := 0
local retval := 0
local key:= lastkey()
if (mode < 3)
   do case
   case (key == K_UP)
      if (mode == 0 .and. line == 1 .and. prevline == 1)
         // [Up] pressed while already on top line -- exit get
         keyboard chr(K_CTRL_W) + chr(K_UP)
      endif
      prevline := line
   case (key == K_DOWN)
      if (mode == 0 .and. line == prevline)
         // [Down] pressed on last non-empty line -- exit get
         keyboard chr(K_CTRL_W) + chr(K_DOWN)
      endif
      prevline := line
   case (key == K_ENTER)
      if (mode == 0 .and. prevline == maxline)
         // [Enter] pressed on last line -- exit get
         keyboard chr(K_CTRL_W) + chr(K_DOWN)
      endif
      prevline := line
   case (mode == 0 .and. key == K_PGDN)
      // [PgDn] pressed - exit get
      keyboard chr(K_CTRL_W) + chr(K_PGDN)
   case (mode == 0 .and. key == K_F10)
      // [F10] pressed - exit get
      keyboard chr(K_CTRL_W) + chr(K_F10)
   case (key == K_CTRL_W)
      prevline := 0
      retval := K_CTRL_W
   case (key == K_ESC)
      prevline := 0
      retval := K_ESC
   case (key == K_DEL .or. key == K_BS .or. key == K_TAB)
      updated := .T.
   case (key >= 32 .and. key <= 255)
      updated := .T.
      if (line == maxline .and. column > maxcol)
         errorwindow(5, 5, "Field is full")
         keyboard chr(K_BS)
      endif
   endcase
endif
return (retval)

/*
--- GEcho 1.12/beta+
 * Origin: The InterACTive BBS - Canberra ACT - (06) 239-7311 (3:620/243)
*/
